/**
  * @file ThermostatARM.c
  * @brief This module is used to control a temperature in HEATING process
  *
  *
  * @author Stefan Popovic
  *
  * @date 24/11/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: STM32F107VC
  * Dev.Board: EasyMx PRO v7
  * Oscillator: HS-PLL 72 MHz
  * Ext. Modules: Thermostat Click Board
  * SW: MikroC PRO for ARM v4.9.0

  * NOTES:
    - Place Thermostat click board on mikroBUS slot 1
    - Set referent temperature via Set_TEMP_REF routine
    - Set temperature hysteresis via Set_TEMP_HYST routine
    - TRES: PC2
    - TE: PD13
    - Timer 2

  /*****************************************************************************
  * Module Preprocessor Constants
  *****************************************************************************/
  #define HYST_TEMP 18
  #define REF_TEMP  22
  #define SLAVE_ADDR 0x48 // Original slave address ( 0x90 ) >> 1

  /*****************************************************************************
  * Module Preprocessor Macros
  *****************************************************************************/


  /*****************************************************************************
  * Module Typedefs
  *****************************************************************************/


  /*****************************************************************************
  * Includes
  *****************************************************************************/
  #include "built_in.h"
  #include "recources.h"
  #include <stdint.h>
  #include <stdbool.h>

  /*****************************************************************************
  * Module Variable Definitions
  *****************************************************************************/
  sbit TEMP at GPIOD_ODR.B13;
  sbit TRES at GPIOC_ODR.B2;
   
  // TFT module connections
  unsigned int TFT_DataPort at GPIOE_ODR;
  sbit TFT_RST at GPIOE_ODR.B8;
  sbit TFT_RS at GPIOE_ODR.B12;
  sbit TFT_CS at GPIOE_ODR.B15;
  sbit TFT_RD at GPIOE_ODR.B10;
  sbit TFT_WR at GPIOE_ODR.B11;
  sbit TFT_BLED at GPIOE_ODR.B9;
  // End TFT module connections

  //variables
  unsigned short temp_MSB;       // Store temperature register MSB byte
  unsigned short temp_LSB;       // Store temperature register LSB byte
  unsigned short old_MSB;
  uint8_t current_temp;
  bool temp_changed;

  void Temp_Read(void);

  void Display_TEMP(void);
  void Set_TEMP_REF(unsigned short);
  void Set_TEMP_HYST(unsigned short);
  void Set_CONFIG(unsigned short);

  char data_[256];
  char txt[24];

  void DrawFrame()
   {
    TFT_Fill_Screen(CL_SILVER);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_SILVER, 0, 0, CL_SILVER, CL_SILVER);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Thermostat  Click  Board  Example", 7, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyMx PRO v7 for STM32 ARM", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Current temperature:", 10, 80 );
    TFT_Write_Text("Relay activates below:", 10, 150 );
    TFT_Write_Text("C", 288, 80);
    TFT_Write_Text("C", 288, 150);
    WordToStr(HYST_TEMP, txt);
    TFT_Write_Text(txt, 230, 150);
    TFT_Set_Brush(1,CL_BLACK, 0, 0, CL_BLACK, CL_BLACK);
    TFT_Circle(275,80,5);
    TFT_Circle(275,150,5);
    TFT_Set_Brush(1,CL_SILVER, 0, 0, CL_SILVER, CL_SILVER);
    TFT_Circle(275,80,3);
    TFT_Circle(275,150,3);
  }

  // Main program
  void main() 
  {
   // MCU Init
   GPIO_Digital_Output(&GPIOA_ODR, _GPIO_PINMASK_ALL);        // Set PORTA as digital output
   GPIO_Digital_Output(&GPIOB_ODR, _GPIO_PINMASK_ALL);        // Set PORTB as digital output
   GPIO_Digital_Output(&GPIOC_ODR, _GPIO_PINMASK_ALL);        // Set PORTC as digital output
   GPIO_Digital_Output(&GPIOD_ODR, _GPIO_PINMASK_ALL);        // Set PORTD as digital output

   GPIOA_ODR = 0;                                             // Set PORTA value to zero
   GPIOB_ODR = 0;                                             // Set PORTB value to zero
   GPIOC_ODR = 0;                                             // Set PORTC value to zero
   GPIOD_ODR = 0;                                             // Set PORTD value to zero
   
   // I2C Init
   I2C1_Init();
   I2C_Set_Active(I2C1_Start, I2C1_Read, I2C1_Write);         // Sets the I2C1 module active

   // TFT Init
   TFT_Init_ILI9341_8bit(320, 240);  // TFT Init
   TFT_BLED = 1;
   DrawFrame();

   // Timer & Interrupt config
   RCC_APB1ENR.TIM2EN = 1;
   TIM2_CR1.CEN = 0;
   TIM2_PSC = 59;
   TIM2_ARR = 59999;
   NVIC_IntEnable(IVT_INT_TIM2);
   TIM2_DIER.UIE = 1;
   TIM2_CR1.CEN = 1;

   //***************************************************************************
   // MAX7502 temperature controller IC

   // MAX7502 Initialization
   TEMP = 0;                  // Thermostat function disabled - Output Relay disconnected
   TRES = 0;                  // Apply Reset procedure
   delay_ms(150);
   TRES = 1;                  // Temperature controller is in default state and ready for work

   // MAX7502 Configuration
   Set_CONFIG(0);             // Set parameters in configuration register
                              // refer to MAX7502 datasheet and click board schematic in order to
                              // set right parameters for intended application:
                              // e.g. set OS output pin polarity for using the chip sensor in
                              // processes of Heating (bit1=0) or Cooling (bit1=1)
                              // or set the output function of the same pin to be Comparator (bit2=0)
                              // or Interrupt logic (bit2=1) in order to use the sensor for hysterezis
                              // temp ON/OFF control (real process) or trivial ON/OFF control
                              // In this example the chip is configured for heating
   Set_TEMP_REF(REF_TEMP);    // Set referent temperature T = 22 C (Output relay is ON until the environment
                              // temperature has reach that value)
   Set_TEMP_HYST(HYST_TEMP);  // Output Relay is OFF until the temperature falls below HYST value

   TEMP = 1;                  // Thermostat enabled - Output Relay connected to OS output pin of MAX7502
   
   //***************************************************************************

   current_temp = REF_TEMP;
   old_MSB = 0;
   temp_MSB = 0;
   temp_changed = false;

   //***************************************************************************
   // Temperature read loop  -  temperature controlling process
   while (1) 
   {
         Temp_read();         // Read 9-bit temperature value from MAX7502's register space
         delay_ms(200);       // +-0.5 C fraction neglected in this example
   }
   //***************************************************************************
}

   void Temp_read()
   {
       I2C1_Start();                                          // I2C START condition
       data_[0] = 0x00;                                       // Write pointer (temperature read only register at address 00h)
       I2C_Write(SLAVE_ADDR, data_, 1, END_MODE_RESTART);     // Send slave address (RD/W = 0), repeated start
       I2C_Read(SLAVE_ADDR, data_, 2, END_MODE_STOP);         //send slave address (RD/W = 1)

       if (old_MSB != current_temp)
           temp_changed = true;
           
       temp_MSB = data_[0];                                   // Read temp MSB byte
       temp_LSB = data_[1];                                   // Read temp LSB byte
       old_MSB = current_temp;
       current_temp = (int8_t) temp_MSB;                      // Set new temperature
       

  }

   void Set_TEMP_REF(unsigned short _temp) 
   {
       // write byte
       I2C1_Start();                                          // I2C START condition
       data_[0] = 0x03;                                       // Set pointer
       data_[1] = _temp;                                      // Set referent temperature
       I2C_Write(SLAVE_ADDR, data_, 2, END_MODE_STOP);        // Send slave address (RD/W = 0)
  }

   void Set_TEMP_HYST(unsigned short _temp)
   {
       // write byte
       I2C1_Start();                                          // I2C START condition
       data_[0] = 0x02;                                       // Set pointer
       data_[1] = _temp;                                      // Set temperature hysterezis
       I2C_Write(SLAVE_ADDR, data_, 2, END_MODE_STOP);        // Send slave address (RD/W = 0)
   }

   void Set_CONFIG(unsigned short _config_value)
   {
       // write byte
       I2C1_Start();                                          // I2C START condition
       data_[0] = 0x01;                                       // Set pointer
       data_[1] = _config_value;                              // Set temperature hysterezis
       I2C_Write(SLAVE_ADDR, data_, 2, END_MODE_STOP);        // Send slave address (RD/W = 0)
   }


   void Timer2_interrupt() iv IVT_INT_TIM2
   {
       TIM2_SR.UIF = 0;

       if (temp_changed)                                      // Draw temperature change
       {
        temp_changed = false;
        TFT_Set_Pen(CL_SILVER, 1);
        TFT_Set_Brush(1,CL_SILVER, 0, 0, CL_SILVER, CL_SILVER);
        TFT_Rectangle(225, 75, 270, 100);
        TFT_Set_Pen(CL_BLACK, 1);
        WordToStr(current_temp, txt);
        TFT_Write_Text(txt, 230, 80);
       }
  }